
-- Webplayer Enabling
local webplayer
if love.web then
	webplayer = true
else
	webplayer = false
end

-- Stuff Used by many different map generator things
local map = {}
local tmap = {}
local run = false
local stept = 1/10
local phase = 0
local steptime = 1/10
local s = 2
local done = false
local sides = {{-1,0},{0,-1},{1,0},{0,1}}
local synced = true
local tileQuads = {}
local tileImages = {}
local tilesetImage
local tilesetBatch
-- Settings
local width = 80
local height = 60

-- Map specific settings
local fillsperstep = 10
local avail = {}
local procent = 0
local t = {}
local maxt = 0

-- Webplayer stuff
local bigfont
local GraphicsUpdate = true
if webplayer then
	function love.run()

		math.randomseed(os.time())
		math.random() math.random()

		if love.load then love.load(arg) end

		local dt = 0

		-- Main loop time.
		while true do
			-- Process events.
			if love.event then
				love.event.pump()
				for e,a,b,c,d in love.event.poll() do
					if e == "quit" then
						if not love.quit or not love.quit() then
							if love.audio then
								love.audio.stop()
							end
							return
						end
					end
					love.handlers[e](a,b,c,d)
				end
			end

			-- Update dt, as we'll be passing it to update
			if love.timer then
				love.timer.step()
				dt = love.timer.getDelta()
			end

			-- Call update and draw
			if love.update then love.update(dt) end -- will pass 0 if love.timer is disabled
			if love.graphics and GraphicsUpdate then
				love.graphics.clear()
				if love.draw then love.draw() end
				GraphicsUpdate = false
			end

			if love.timer then love.timer.sleep(0.001) end
			if love.graphics then love.graphics.present() end

		end

	end
else
	bigfont = love.graphics.newFont(64)
end

function math.mod(a,b) -- Doesn't appear to work in webplayer otherwise
	return a - math.floor(a/b)*b
end

local function bin(left,up,right,down) -- To get the ID of the appropiate image
	local i = 1
	if left then i = i + 1 end
	if up then i = i + 2 end
	if right then i = i + 4 end
	if down then i = i + 8 end
	return i
end

local function debin(b) -- From ID to where it goes
	b = b - 1
	local left,up,right,down = false,false,false,false
	if math.mod(b,8) ~= b then
		down = true
		b = math.mod(b,8)
	end
	if math.mod(b,4) ~= b then
		right = true
		b = math.mod(b,4)
	end
	if math.mod(b,2) ~= b then
		up = true
		b = math.mod(b,2)
	end
	if math.mod(b,1) ~= b then
		left = true
		b = math.mod(b,1)
	end
	return left,up,right,down
end

function love.load()
	-- Initiate Map with width x height
	for i = 1,height do
		map[i] = {}
		tmap[i] = {}
		for j = 1,width do
			map[i][j] = 1
			tmap[i][j] = 0
		end
	end

	-- Image loading / creating
	for i = 1,5 do
		if webplayer then
			tileImages[i] = love.graphics.newImage("tileImage" .. i .. ".jpg")
			if tileImages[i] == nil then
				tileImages[i] = love.graphics.newImage("weavemaze/tileImage" .. i .. ".jpg")
			end
		else
			tileImages[i] = createImage(i)
		end
	end
	tilesetImage = tileImages[s]
	
	-- Creating tileQuads
	for i = 1,18 do
		tileQuads[i] = love.graphics.newQuad(i*10-10,0, 10, 10,tilesetImage:getWidth(), tilesetImage:getHeight())
	end
	
	-- tilesetBatch
	if not webplayer then
		tilesetBatch = love.graphics.newSpriteBatch(tilesetImage, width * height)
		--updateTiles()
	end
end

if not webplayer then -- Doesnt need if it's not webplayer
	function createImage(s)
		local tilesetImage = love.image.newImageData(180,10)
		for h = 1,16 do
			local left,up,right,down = debin(h)
			for i = 0,9 do
				for j = 0,9 do
					local r,g,b,a = 255,255,255,255
					if (i == s-1 and j == s-1) or (i == s-1 and j == 10-s) or (i == 10-s and j == 10-s) or (i == 10-s and j == s-1) then
						r,g,b = 0,0,0
					elseif (i == s-1 or i == 10-s) and j < s-1 then
						if up then
							r,g,b = 0,0,0
						else
							r,g,b = 127,127,127
						end
					elseif (i == s-1 or i == 10-s) and j > 10-s then
						if down then
							r,g,b = 0,0,0
						else
							r,g,b = 127,127,127
						end
					elseif (j == s-1 or j == 10-s) and i < s-1 then
						if left then
							r,g,b = 0,0,0
						else
							r,g,b = 127,127,127
						end
					elseif (j == s-1 or j == 10-s) and i > 10-s then
						if right then
							r,g,b = 0,0,0
						else
							r,g,b = 127,127,127
						end
					elseif i < s and j < s then
						r,g,b = 127,127,127
					elseif i < s and j > 9-s then
						r,g,b = 127,127,127
					elseif i > 9-s and j < s then
						r,g,b = 127,127,127
					elseif i > 9-s and j > 9-s then
						r,g,b = 127,127,127
					elseif i > s-1 and i < 10-s and j == s-1 then
						if not up then
							r,g,b = 0,0,0
						end
					elseif i > s-1 and i < 10-s and j == 10-s then
						if not down then
							r,g,b = 0,0,0
						end
					elseif j > s-1 and j < 10-s and i == s-1 then
						if not left then
							r,g,b = 0,0,0
						end
					elseif j > s-1 and j < 10-s and i == 10-s then
						if not right then
							r,g,b = 0,0,0
						end
					elseif i > s-1 and i < 10-s and j < s-1 then
						if not up then
							r,g,b = 127,127,127
						end
					elseif i > s-1 and i < 10-s and j > 10-s then
						if not down then
							r,g,b = 127,127,127
						end
					elseif j > s-1 and j < 10-s and i < s-1 then
						if not left then
							r,g,b = 127,127,127
						end
					elseif j > s-1 and j < 10-s and i > 10-s then
						if not right then
							r,g,b = 127,127,127
						end
					end
					tilesetImage:setPixel(i+h*10-10,j,r,g,b,a)
				end
			end
			tileQuads[h] = love.graphics.newQuad( h*10-10, 0, 10, 10, 180, 10 )
		end
		for i = 0,9 do
			for j = 0,9 do
				tilesetImage:setPixel(i+160,j,127,127,127,255)
				if i == s-1 or i == 10-s then
					tilesetImage:setPixel(i+160,j,0,0,0,255)
				elseif (j == s-1 or j == 10-s) and (i < s-1 or i > 10-s) then
					tilesetImage:setPixel(i+160,j,0,0,0,255)
				elseif (i > s-1 and i < 10-s) or (j > s-1 and j < 10-s) then
					tilesetImage:setPixel(i+160,j,255,255,255,255)
				end
				
				tilesetImage:setPixel(i+170,j,127,127,127,255)
				if j == s-1 or j == 10-s then
					tilesetImage:setPixel(i+170,j,0,0,0,255)
				elseif (i == s-1 or i == 10-s) and (j < s-1 or j > 10-s) then
					tilesetImage:setPixel(i+170,j,0,0,0,255)
				elseif (i > s-1 and i < 10-s) or (j > s-1 and j < 10-s) then
					tilesetImage:setPixel(i+170,j,255,255,255,255)
				end
			end
		end
		tilesetImage = love.graphics.newImage(tilesetImage)
		return tilesetImage
	end
end

local function table_concat(t1,t2) -- Because there's no function that does this in the lua API
	for i,v in ipairs(t2) do
		table.insert(t1,v)
		t2[i] = nil
	end
	return t1
end

local function updateTiles() -- Update Tiles
	tilesetBatch:clear()
	for y = 1,height do
		for x = 1,width do
			if map[y][x] ~= 0 then
				tilesetBatch:addq(tileQuads[math.min(map[y][x],18)], x*10-10, y*10-10)
			end
		end
	end
end

local function move(n) -- where an image with ID n goes to at average
	local l,u,r,d = debin(n)
	local p = {x=0,y=0}
	if l then p.x = p.x - 1 end
	if u then p.y = p.y - 1 end
	if r then p.x = p.x + 1 end
	if d then p.y = p.y + 1 end
	return p
end

local function switch(s) -- switches left with right and up with down
	if s > 4 then
		return (s-1)/4+1
	else
		return (s-1)*4+1
	end
end

local function absorb(g,d) -- Changes all of one t to another t
	for y = 1,height do
		for x = 1,width do
			if tmap[y][x] == d then
				tmap[y][x] = g
			end
		end
	end
end

local function check(x,y,t) -- If it's a good point to merge with
	if x > 0 and x < width+1 and y > 0 and y < height+1 then
		if map[y][x] == 1 then
			return true
		elseif tmap[y][x] ~= t then
			return true
		else
			return false
		end
	else
		return false
	end
end

local function extend(x,y,t,n) -- Merge 2 points
	-- just makes sure it's fixed correctly
	if (tmap[y][x] == tmap[y+move(n).y][x+move(n).x] and tmap[y][x] ~= 0) or map[y][x] > 16 or map[y+move(n).y][x+move(n).x] > 16 then
		print("missed one")
	else
		map[y][x] = map[y][x] + n - 1
		map[y+move(n).y][x+move(n).x] = map[y+move(n).y][x+move(n).x] + switch(n) - 1
		tmap[y][x] = t
		if tmap[y+move(n).y][x+move(n).x] == 0 then
			tmap[y+move(n).y][x+move(n).x] = t
		else
			absorb(tmap[y][x],tmap[y+move(n).y][x+move(n).x])
		end
	end
end

local function step() -- Step()
	if phase == 0 then
		local p = 1
		for i = 1, height do
			for j = 1, width do
				tmap[i][j] = p
				p = p + 1
			end
		end
		for i = 1, height do
			for j = 1, width do
				if i ~= height then
					table.insert(t,{x=j,y=i,n=bin(false,false,false,true)})
				end
				if j ~= width then
					table.insert(t,{x=j,y=i,n=bin(false,false,true,false)})
				end
			end
		end
		maxt = #t
	end
	if #t == 0 then
		done = true
		run = false
		synced = false
	else
		for i = 1,math.min(fillsperstep,#t) do
			local c = math.max(math.ceil(math.random()*#t),1)
			--print(t[c].x,t[c].y,t[c].t,t[c].n)
			--print(t[c].x+move(t[c].n).x,t[c].y+move(t[c].n).y)
			if tmap[t[c].y][t[c].x] == tmap[t[c].y+move(t[c].n).y][t[c].x+move(t[c].n).x] then
			else
				extend(t[c].x,t[c].y,tmap[t[c].y][t[c].x],t[c].n)
				if not webplayer then
					updateTiles()
				end
			end
			table.remove(t,c)
			procent = 100 - math.ceil(#t / maxt * 100)
		end
	end
	phase = phase + 1 -- Next phase
end

local bugging = false

function love.keypressed(k)
	if k == "return" then
		run = not run
		stept = 0
	elseif k == " " then
		step()
	elseif k == "f1" then
		s = math.mod(s,4) + 1
		tilesetImage = tileImages[s]
		if not webplayer then
			tilesetBatch = nil
			tilesetBatch = love.graphics.newSpriteBatch(tilesetImage, width * height)
			updateTiles()
		end
	elseif k == "f2" then
		synced = not synced
	end
end

function love.update(dt)
	if not done then
		if not synced then
			step()
		elseif run then
			stept = stept + dt
			if stept >= steptime then
				step()
				stept = stept - steptime
			end
			stept = math.mod(stept,steptime*4)
		end
	end
end

function love.draw()
	love.graphics.setColor(255,255,255)
	if webplayer then
		for y = 1,height do
			for x = 1,width do
				love.graphics.drawq( tilesetImage, tileQuads[map[y][x]], x*10-10, y*10-10)
			end
		end
	else
		love.graphics.draw(tilesetBatch,0,0,0,1,1)
	end
	love.graphics.setColor(0,0,255)
	if not webplayer then
		love.graphics.setFont(bigfont)
	end
	if phase == 0 then
		love.graphics.print("Spacebar to step()",5,5)
		love.graphics.print("Enter to auto step()",5,70)
	elseif not done then
		love.graphics.print(phase,0,0)
	end
	love.graphics.print(procent .. "%",5,531)
	love.graphics.setColor(255,255,255)
	if webplayer then
		love.graphics.print("http://zer0.team.pro",5,583)
	end
end
